@@ -29,7 +29,6 @@
 (require 'haskell-repl)
 (require 'haskell-load)
 (require 'haskell-commands)
-(require 'haskell-sandbox)
 (require 'haskell-modules)
 (require 'haskell-string)
 (require 'haskell-completions)
@@ -45,13 +44,10 @@
     (define-key map (kbd "M-.") 'haskell-mode-jump-to-def-or-tag)
     (define-key map (kbd "C-c C-k") 'haskell-interactive-mode-clear)
     (define-key map (kbd "C-c C-c") 'haskell-process-cabal-build)
-    (define-key map (kbd "C-c v c") 'haskell-cabal-visit-file)
+    (define-key map (kbd "C-c C-v") 'haskell-cabal-visit-file)
     (define-key map (kbd "C-c C-x") 'haskell-process-cabal)
     (define-key map (kbd "C-c C-b") 'haskell-interactive-switch)
     (define-key map (kbd "C-c C-z") 'haskell-interactive-switch)
-    (define-key map (kbd "M-n") 'haskell-goto-next-error)
-    (define-key map (kbd "M-p") 'haskell-goto-prev-error)
-    (define-key map (kbd "C-c M-p") 'haskell-goto-first-error)
     map)
   "Keymap for using `interactive-haskell-mode'.")
 
@@ -135,7 +131,6 @@
 ;;;###autoload
 (defun haskell-session-kill (&optional leave-interactive-buffer)
   "Kill the session process and buffer, delete the session.
-0. Prompt to kill all associated buffers.
 1. Kill the process.
 2. Kill the interactive buffer unless LEAVE-INTERACTIVE-BUFFER is not given.
 3. Walk through all the related buffers and set their haskell-session to nil.
@@ -144,12 +139,7 @@
   (haskell-mode-toggle-interactive-prompt-state)
   (unwind-protect
       (let* ((session (haskell-session))
-             (name (haskell-session-name session))
-             (also-kill-buffers
-              (and haskell-ask-also-kill-buffers
-                   (y-or-n-p
-                    (format "Killing `%s'. Also kill all associated buffers?"
-                            name)))))
+             (name (haskell-session-name session)))
         (haskell-kill-session-process session)
         (unless leave-interactive-buffer
           (kill-buffer (haskell-session-interactive-buffer session)))
@@ -158,16 +148,18 @@
                       (when (and (boundp 'haskell-session)
                                  (string= (haskell-session-name haskell-session)
                                           name))
-                        (setq haskell-session nil)
-                        (when also-kill-buffers
-                          (kill-buffer)))))
+                        (setq haskell-session nil))))
         (setq haskell-sessions
               (cl-remove-if (lambda (session)
                               (string= (haskell-session-name session)
                                        name))
-                            haskell-sessions)))
+                            haskell-sessions))
+        (run-hooks 'haskell-session-kill-hook))
     (haskell-mode-toggle-interactive-prompt-state t)))
 
+
+
+
 ;;;###autoload
 (defun haskell-interactive-kill ()
   "Kill the buffer and (maybe) the session."
@@ -177,12 +169,12 @@
     (unwind-protect
         (when (and (boundp 'haskell-session)
                    haskell-session
-                   (y-or-n-p "Kill the whole session?"))
+                   (y-or-n-p "Kill the whole session? "))
           (haskell-session-kill t)))
     (haskell-mode-toggle-interactive-prompt-state t)))
 
 (defun haskell-session-make (name)
-  "Make a Haskell session."
+  "Make a Haskell session called NAME."
   (when (haskell-session-lookup name)
     (error "Session of name %s already exists!" name))
   (let ((session (setq haskell-session
@@ -249,7 +241,8 @@ If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
 
 (defun haskell-process-prompt-restart (process)
   "Prompt to restart the died PROCESS."
-  (let ((process-name (haskell-process-name process)))
+  (let ((process-name (haskell-process-name process))
+        (cursor-in-echo-area t))
     (if haskell-process-suggest-restart
         (progn
           (haskell-mode-toggle-interactive-prompt-state)
@@ -257,7 +250,7 @@ If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
               (cond
                ((string-match "You need to re-run the 'configure' command."
                               (haskell-process-response process))
-                (cl-case (read-event
+                (cl-case (read-char-choice
                           (concat
                            "The Haskell process ended. Cabal wants you to run "
                            (propertize "cabal configure"
@@ -268,7 +261,8 @@ If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
                            "Cabal said:\n\n"
                            (propertize (haskell-process-response process)
                                        'face
-                                       'font-lock-comment-face)))
+                                       'font-lock-comment-face))
+                          '(?l ?n ?y))
                   (?y (let ((default-directory
                               (haskell-session-cabal-dir
                                (haskell-process-session process))))
@@ -283,12 +277,13 @@ If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
                                  (insert response)))))
                   (?n)))
                (t
-                (cl-case (read-event
+                (cl-case (read-char-choice
                           (propertize
-                           (format "The Haskell process `%s' has died. Restart? (y, n, l: show process log)"
+                           (format "The Haskell process `%s' has died. Restart? (y, n, l: show process log) "
                                    process-name)
                            'face
-                           'minibuffer-prompt))
+                           'minibuffer-prompt)
+                          '(?l ?n ?y))
                   (?y (haskell-process-start (haskell-process-session process)))
                   (?l (let* ((response (haskell-process-response process))
                              (buffer (get-buffer "*haskell-process-log*")))
@@ -300,17 +295,12 @@ If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
                   (?n))))
             ;; unwind
             (haskell-mode-toggle-interactive-prompt-state t)))
-      (message
-       (format "The Haskell process `%s' is dearly departed." process-name)))))
+      (message "The Haskell process `%s' is dearly departed." process-name))))
 
 (defun haskell-process ()
   "Get the current process from the current session."
   (haskell-session-process (haskell-session)))
 
-(defun haskell-interactive-buffer ()
-  "Get the interactive buffer of the session."
-  (haskell-session-interactive-buffer (haskell-session)))
-
 ;;;###autoload
 (defun haskell-kill-session-process (&optional session)
   "Kill the process."
@@ -344,7 +334,7 @@ If `haskell-process-load-or-reload-prompt' is nil, accept `default'."
 Give optional NEXT-P parameter to override value of
 `xref-prompt-for-identifier' during definition search."
   (interactive "P")
-  (let ((ident (haskell-ident-at-point))
+  (let ((ident (haskell-string-drop-qualifier (haskell-ident-at-point)))
         (tags-file-dir (haskell-cabal--find-tags-dir))
         (tags-revert-without-query t))
     (when (and ident
@@ -359,12 +349,8 @@ Give optional NEXT-P parameter to override value of
 ;;;###autoload
 (defun haskell-mode-after-save-handler ()
   "Function that will be called after buffer's saving."
-  (when haskell-tags-on-save (ignore-errors (haskell-mode-generate-tags)))
-  (when haskell-stylish-on-save
-    (ignore-errors (haskell-mode-stylish-buffer))
-    (let ((before-save-hook '())
-          (after-save-hook '()))
-      (basic-save-buffer))))
+  (when haskell-tags-on-save
+    (ignore-errors (haskell-mode-generate-tags))))
 
 ;;;###autoload
 (defun haskell-mode-tag-find (&optional _next-p)
@@ -504,7 +490,7 @@ for various things, but is optional."
   (insert-file-contents-literally
    (concat (haskell-session-current-dir (haskell-session))
            "/"
-           (haskell-guess-module-name)
+           (haskell-guess-module-name-from-file-name (buffer-file-name))
            ".imports")))
 
 (defun haskell-interactive-jump-to-error-line ()
